#
# Licensed Materials - Property of IBM
# Rational ClearQuest
# (C) Copyright IBM Corp. 2004, 2014. All Rights Reserved.
#
# US Government Users Restricted Rights -
# Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
#
#
# Description:
#
# This script is used to enable/disable LDAP authentication for a user.  The usage 
# is listed below and can be output with the -HELP argument to this script.  The
# script requires a clearquest dbset that has been LDAP enabled (see LDAP documentation).
# It is also required that the user being LDAP enabled exists in the ClearQuest 
# database as well as in LDAP.  The following are examples of the usage:
#
# example 1. Make CQ user anyperson LDAP enabled:
# ratlperl SetupCQLDAP.pl -MODE LDAP -CQUSer anyperson -cqpassword "" -ldapuser 
# anyperson@us.ibm.com -dbset ldap2 -cqadminuser admin -cqadminpassword ""
#
#
# example 2. Disable LDAP authentication for CQ user anyperson:
# ratlperl SetupCQLDAP.pl -MODE CQ -CQUSer anyperson -cqpassword "" -dbset ldap2 
# -cqadminuser admin -cqadminpassword ""
#
################################################################################
# Usage:
# ratlperl SetupCQLDAP [-HELP] |
#                      [-MODE <CQ|LDAP>]
#                      [-CQUSER <CQ user>]
#                      [-CQPASSWORD <CQ user's password>]
#                      [-LDAPUSER <LDAP user>]
#                      [-DBSET <dbset>]
#                      [-CQADMINUSER <CQ admin user>]
#                      [-CQADMINPASSWORD <CQ admin pswd>]
#
################################################################################

# Load the CQ API interface...
use CQPerlExt;

use strict;

# Define default values...
my(%ARG) = (
    MODE => "LDAP",
    CQPASSWORD => "",
    LDAPUSER => "",
    DBSET => "2003.06.00",
    CQADMINUSER => "admin",
    CQADMINPASSWORD => "",
);

# Define command syntax...
my($Usage) = "Usage:\n"
    ."\n"
    ."  ratlperl SetupCQLDAP.pl [-HELP] |\n"
    ."                        [-MODE <CQ|LDAP>]\n"
    ."                        [-CQUSER <CQ user>]\n"
    ."                        [-CQPASSWORD <CQ user's password>]\n"
    ."                        [-LDAPUSER <LDAP user>]\n"
    ."                        [-DBSET <dbset>]\n"
    ."                        [-CQADMINUSER <CQ admin user>]\n"
    ."                        [-CQADMINPASSWORD <CQ admin pswd>]\n"
    ."\n"
    ."  Parameters:\n"
    ."    -HELP             Displays this usage message\n"
    ."    -MODE:            Authentication mode to set (CQ or LDAP default=$ARG{MODE})\n"
    ."    -CQUSER:          CQ user whose mode is to be set (default='$ARG{CQUSER}')\n"
    ."    -CQPASSWORD:      CQ user's password (only required if '-MODE CQ')\n"
    ."    -LDAPUSER:        LDAP user to which this CQ user will be associated if\n"
    ."                      '-MODE LDAP' (no default value -- must specify)\n"
    ."    -DBSET:           CQ DBSet (connection) name (default='$ARG{DBSET}')\n"
    ."    -CQADMINUSER:     CQ administrative user (default='$ARG{CQADMINUSER}')\n"
    ."    -CQADMINPASSWORD: CQ administrative user's password (default='$ARG{CQADMINPASSWORD}')\n"
;

# Parse the user's command-line parameters...
while (@ARGV) {
    my($A) = shift;
    if ($A =~ /-(MODE|CQUSER|CQPASSWORD|LDAPUSER|DBSET|CQADMINUSER|CQADMINPASSWORD)/i) {
        # Keyword/value parameter; get the value...
        $ARG{uc $1} = shift;
    } elsif ($A =~ /-(NO|)(HELP)/i) {
        # "Switch" argument... set it to TRUE or FALSE depending on whether
        # it was "-NOxxx" or "-xxx"...
        $ARG{uc $2} = ((uc $1) eq "NO") ? 0 : 1;
    } else {
        die "ERROR: Unrecognized argument '$A'.";
    }
}

# See if they requested usage info...
if ($ARG{HELP}) {
    print $Usage;
    exit;
}

# Verify that we have all required args...
my(@Errors);
push @Errors, "ERROR: Missing parameter 'MODE'" if (not $ARG{MODE});
$ARG{MODE} = uc $ARG{MODE};
if (not $ARG{MODE} =~ /(CQ|LDAP)/i) {
    push @Errors, "ERROR: -MODE must be 'CQ' or 'LDAP'";
}
push @Errors, "ERROR: Missing parameter 'CQUSER'" if (not $ARG{CQUSER});
push @Errors, "ERROR: Missing parameter 'CQADMINUSER'" if (not $ARG{CQADMINUSER});
if (@Errors) {
    print join("\n", @Errors)
        ."\n\n$Usage"
    ;
    exit 1;
}

# Tell the user what we're going to do...
print "Setting CQ user to '$ARG{MODE}' authentication mode:\n"
    ."  CQ user:                '$ARG{'CQUSER'}'\n"
    .(($ARG{MODE} eq "CQ") ? "  CQ password:            '$ARG{'CQPASSWORD'}'\n" : "")
    .(($ARG{MODE} eq "LDAP" and $ARG{'LDAPUSER'}) ? "  LDAP user:              '$ARG{'LDAPUSER'}'\n" : "")
    ."  DBSet:                  '$ARG{'DBSET'}'\n"
    ."  CQ admin user:          '$ARG{'CQADMINUSER'}'\n"
    ."  CQ admin password:      '$ARG{'CQADMINPASSWORD'}'\n"
    ."\n"
;

# Table of authentication modes...
my(%AuthModes) = (
    $CQPerlExt::CQ_LDAP_AUTHENTICATION => "CQ_LDAP_AUTHENTICATION",
    $CQPerlExt::CQ_CQ_AUTHENTICATION => "CQ_CQ_AUTHENTICATION",
);

# Create and log on to an Admin session...
my($CQAdminSession);
eval {
    $CQAdminSession = CQAdminSession::Build();
    $CQAdminSession->Logon(
        $ARG{'CQADMINUSER'}, $ARG{'CQADMINPASSWORD'}, $ARG{'DBSET'}
    );
};
die "ERROR logging on to AdminSession: $@" if ($@);

# Get the user's CQUser object...
my($CQUser);
eval { $CQUser = $CQAdminSession->GetUser($ARG{'CQUSER'}); };
die "ERROR getting user '$ARG{'CQUSER'}'s CQUser object: $@" if ($@);

if (! $CQUser){
		die "Could not find Clearquest user $ARG{'CQUSER'}...Exiting. \n";
}


# Get and show the user's current authentication mode...
my($AuthMode);
eval { $AuthMode = $CQUser->GetAuthenticationMode(); };
die "ERROR getting user '$ARG{'CQUSER'}'s prior authentication mode: $@" if ($@);
print "User '$ARG{'CQUSER'}'s prior authentication mode was $AuthModes{$AuthMode}.\n";

my($NewLogin);
if ($ARG{MODE} eq "CQ") {
    eval { $CQUser->SetCQAuthentication($ARG{CQPASSWORD}); };
    die "ERROR setting user '$ARG{'CQUSER'}'s authentication mode to CQ: $@" if ($@);
    $NewLogin = "  Userid: '$ARG{CQUSER}'\n  Password: '$ARG{CQPASSWORD}'";
} else {
    # LDAP mode requested...Set the user to LDAP authentication mode...
    eval { $CQUser->SetLDAPAuthentication($ARG{LDAPUSER}); };
    die "ERROR setting user '$ARG{'CQUSER'}'s authentication mode to LDAP"
        ." using LDAP user '$ARG{LDAPUSER}': $@" if ($@);

    # Tell the user that they can now log on using LDAP authentication mode.
    # NOTE: Telling them what userid to use is a little tricky; it's the LDAP
    # attribute, and we don't have direct access to the LDAP database to know
    # what the attribute value is supposed to be. So figure it out by applying
    # the following rules:
    # 1. If the user specified an LDAP userid when they invoked us, then that's
    #    the one to show them.
    # 2. If the user did not specify an LDAP userid, then see if we can figure
    #    out which CQ user field maps to LDAP for logon purposes, and pull the
    #    value out of CQ. This is possible if:
    #    (a) The LDAP configuration uses the "optimized" mapping feature (second
    #        operand to setlapmap is '%login%').
    #    (b) The LDAP configuration uses the same LDAP attribute for searching
    #        (logon) and mapping (matching up corresponding entries between CQ
    #        and LDAP).
    #    For either of these cases, we can get the value from the CQ userid from
    #    the field that comprises the "CQ side" of the CQ<=>LDAP mapping. For
    #    any other type of CQ<=>LDAP mapping, it's not possible for us to
    #    determine what LDAP userid corresponds to this CQ userid, so we'll just
    #    suppress the informational message...
    my $ldapuser = $ARG{LDAPUSER};
    if (! $ldapuser) {
        eval {
            # Get info about the LDAP configuration...
            my($LDAPLoginAttribute, $CQLDAPMapCQ, $CQLDAPMapLDAP, $CMD);
            $CMD = join('', `installutil getldapsearch \"$ARG{'DBSET'}\" \"$ARG{'CQADMINUSER'}\" \"$ARG{'CQADMINPASSWORD'}\"`);
            if ($CMD =~ /.*?(\w+)=\%login\%/si) {
                $LDAPLoginAttribute = $1;
            } else {
                die "ERROR: Unable to retrieve LDAP login attribute; getldapsearch output:\n$CMD";
            }
            $CMD = join('', `installutil getcqldapmap \"$ARG{'DBSET'}\" \"$ARG{'CQADMINUSER'}\" \"$ARG{'CQADMINPASSWORD'}\"`);
            if ($CMD =~ /.*?CQUser\.(\w+) <-+> ([\w\%]+)/si) {
                ($CQLDAPMapCQ, $CQLDAPMapLDAP) = ($1, $2);
            } else {
                die "ERROR: Unable to retrieve CQ-LDAP mapping information; getcqldapmap output:\n$CMD";
            }
            # If the CQ<=>LDAP mapping is set up such that it's possible to get
            # the LDAP-userid value from CQ, then lets do it; but if it isn't,
            # then we can't...
            if ( ($CQLDAPMapLDAP =~ /^\%login\%$/i)
                or ($CQLDAPMapLDAP eq $LDAPLoginAttribute) ) {
                # It's mapped CQ<=>LDAP using the same attribute that's
                # used for logging on; therefore we can get the value
                # of the corresponding attribute of the CQ user...
                if ($CQLDAPMapCQ eq "CQ_LOGIN_NAME") {
                    $ldapuser = $ARG{'CQUSER'};
                } elsif ($CQLDAPMapCQ eq "CQ_FULLNAME") {
                    $ldapuser = $CQUser->GetFullName();
                } elsif ($CQLDAPMapCQ eq "CQ_EMAIL") {
                    $ldapuser = $CQUser->GetEmail();
                } elsif ($CQLDAPMapCQ eq "CQ_PHONE") {
                    $ldapuser = $CQUser->GetPhone();
                } elsif ($CQLDAPMapCQ eq "CQ_MISC_INFO") {
                    $ldapuser = $CQUser->GetMiscInfo();
                } else {
                    die "ERROR: Unrecognized CQ LDAP Mapping Field value '$CQLDAPMapCQ'";
                }
            }
        };
        die "ERROR: Unable to retrieve LDAP userid information for CQ user '$ARG{'CQUSER'}': '$@'"
            if ($@);
    }
    $NewLogin = "  Userid: $ldapuser\n  Password: (that user's LDAP password)\n"
        if ($ldapuser);
}

# Get and show the user's new authentication mode (to make sure it was
# really set as we want)...
my($AuthMode);
eval { $AuthMode = $CQUser->GetAuthenticationMode(); };
die "ERROR getting user '$ARG{'CQUSER'}'s new authentication mode: $@" if ($@);
print "User '$ARG{'CQUSER'}'s new authentication mode is $AuthModes{$AuthMode}.\n";

# Upgrade all user databases to reflect this change...
my($BadUpgradeAREF);
eval { $BadUpgradeAREF = $CQUser->UpgradeInfo(); };
die "ERROR upgrading user databases: $@" if ($@);
if (@$BadUpgradeAREF) {
    print "WARNING: The following databases failed to upgrade properly: "
        .join(", ", @$BadUpgradeAREF)
        ."\n"
    ;
}

# Clean up all our CQ objects...
undef $CQUser;
undef $CQAdminSession;

print "\nOperation complete.\n";
print "You may now log on to CQ using:\n".$NewLogin."\n" if ($NewLogin);
